pub mod directory_processor;
pub mod error;
pub mod file_type;
pub mod logging;
pub mod tools;

// Constants for file processing
pub const FILES_TO_CONVERT: [&str; 3] = ["odt", "doc", "docx"];
pub const FILES_TO_COPY: [&str; 9] = [
    "jpg", "jpeg", "png", "gif", "bmp", "tiff", "webp", "avif", "txt",
];
pub const PATHS_TO_IGNORE: [&str; 5] = [
    ".DS_Store",
    ".syncthing",
    ".sync-conflict-",
    ".stfolder",
    ".stversions",
];

// Re-export commonly used items
pub use crate::logging::LogConfig;
pub use log::{debug, error, info, warn};

/// Macro for logging with file and line information
#[macro_export]
macro_rules! log_detail {
    ($level:expr, $($arg:tt)+) => {
        log::log!($level, "{} - {}", file!(), format!($($arg)+))
    };
}

/// Macro for performance logging with timing information
#[macro_export]
macro_rules! log_timed {
    ($level:expr, $operation:expr, $block:expr) => {{
        let start = std::time::Instant::now();
        let result = $block;
        let duration = start.elapsed();
        log::log!($level, "{} completed in {:.2}ms", $operation, duration.as_secs_f64() * 1000.0);
        result
    }};
}

#[cfg(all(test, feature = "test-sync"))]
mod tests {
    use super::*;
    use crate::error::Result;
    use crate::logging::{LogConfig, init_logging};
    use log::LevelFilter;
    use std::fs;
    use tempfile::TempDir;

    fn setup_test_log_file() -> (TempDir, std::path::PathBuf) {
        let temp_dir = tempfile::TempDir::new().expect("Failed to create temporary directory for test");
        let log_path = temp_dir.path().join("test.log");
        (temp_dir, log_path)
    }

    #[test]
    fn test_logging_macros() -> Result<()> {
        let (_temp_dir, log_path) = setup_test_log_file();

        let config = LogConfig {
            log_file: Some(log_path.clone()),
            log_level: LevelFilter::Debug,
            append_log: false,
        };

        init_logging(config)?;

        log_detail!(log::Level::Info, "test detail message");
        log_timed!(log::Level::Info, "test operation", {
            std::thread::sleep(std::time::Duration::from_millis(10));
        });

        let content = fs::read_to_string(&log_path)?;
        assert!(content.contains("test detail message"));
        assert!(content.contains("test operation completed in"));
        assert!(content.contains("ms"));

        Ok(())
    }
}
