use log::{LevelFilter, Log, Record, Metadata};
use std::fs::OpenOptions;
use std::io::Write;
use chrono::Local;
use std::sync::{Mutex, Arc};
use std::sync::OnceLock;
use crate::error::{LogError, Result};
use super::prod_logger::{LogConfig, LogWriter};

#[derive(Debug)]
struct TestLogger {
    writer: Arc<Mutex<LogWriter>>,
    level: Arc<Mutex<LevelFilter>>,
}

impl TestLogger {
    fn new(writer: Arc<Mutex<LogWriter>>, level: Arc<Mutex<LevelFilter>>) -> Self {
        Self { writer, level }
    }

    fn get_level(&self) -> Result<LevelFilter> {
        self.level
            .lock()
            .map_err(|e| LogError::LockPoisoned(e.to_string()).into())
            .map(|guard| *guard)
    }

    fn write_log(&self, message: &str) -> Result<()> {
        self.writer
            .lock()
            .map_err(|e| LogError::LockPoisoned(e.to_string()).into())
            .and_then(|mut writer| {
                writer.write_all(message.as_bytes())
                    .and_then(|_| writer.flush())
                    .map_err(|e| LogError::Io(e).into())
            })
    }
}

impl Log for TestLogger {
    fn enabled(&self, metadata: &Metadata) -> bool {
        self.get_level().map(|level| metadata.level() <= level).unwrap_or(false)
    }

    fn log(&self, record: &Record) {
        if !self.enabled(record.metadata()) {
            return;
        }

        let message = format!(
            "{} [{:<5}] - {}\n",
            Local::now().format("%Y-%m-%d %H:%M:%S%.3f"),
            record.level(),
            record.args()
        );

        if let Err(e) = self.write_log(&message) {
            eprintln!("Failed to write log message: {}", e);
        }
    }

    fn flush(&self) {
        if let Err(e) = self.writer
            .lock()
            .map_err(|e| LogError::LockPoisoned(e.to_string()))
            .and_then(|mut w| w.flush().map_err(LogError::Io))
        {
            eprintln!("Failed to flush logger: {}", e);
        }
    }
}

static LOGGER: OnceLock<Arc<TestLogger>> = OnceLock::new();
static LEVEL: OnceLock<Arc<Mutex<LevelFilter>>> = OnceLock::new();

/// Initialize logging with enhanced features for testing
///
/// # Errors
///
/// Returns an error if:
/// - Failed to create log directory
/// - Failed to open log file
/// - Failed to write initial log header
/// - Failed to set global logger
/// - Logger is already initialized
pub fn init_test_logging(config: LogConfig) -> Result<()> {
    let log_file = if let Some(log_path) = &config.log_file {
        // Create parent directory if it doesn't exist
        if let Some(parent) = log_path.parent() {
            std::fs::create_dir_all(parent).map_err(LogError::Io)?;
        }

        let file = OpenOptions::new()
            .create(true)
            .write(true)
            .append(config.append_log)
            .truncate(!config.append_log)
            .open(log_path)
            .map_err(LogError::Io)?;

        // Write header for new log files
        if !config.append_log {
            writeln!(
                &file,
                "=== Log started at {} ===",
                Local::now().format("%Y-%m-%d %H:%M:%S")
            ).map_err(LogError::Io)?;
        }
        Some(file)
    } else {
        None
    };

    let level = LEVEL
        .get_or_init(|| Arc::new(Mutex::new(config.log_level)))
        .clone();
    
    {
        let mut lvl = level.lock()
            .map_err(|e| LogError::LockPoisoned(e.to_string()))?;
        *lvl = config.log_level;
    }

    let writer = Arc::new(Mutex::new(LogWriter::new(log_file)));
    let logger = Arc::new(TestLogger::new(writer.clone(), level.clone()));

    // Try to set the global logger
    if LOGGER.get().is_some() {
        return Err(LogError::AlreadyInitialized.into());
    }

    // Set the logger and store it
    log::set_boxed_logger(Box::new(TestLogger::new(writer, level.clone())))
        .map_err(|e| LogError::Init(e.to_string()))?;

    // Store our logger instance
    if LOGGER.set(logger).is_err() {
        return Err(LogError::AlreadyInitialized.into());
    }

    log::set_max_level(config.log_level);
    Ok(())
} 